local MAJOR = "StarLibRGBA-1.0" 
local MINOR = 12

assert(LibStub, MAJOR.." requires LibStub") 
local LibRGBA = LibStub:NewLibrary(MAJOR, MINOR)
if not LibRGBA then return end

local pool = setmetatable({}, {__mode = "k"})

if not LibRGBA.__index then
	LibRGBA.__index = LibRGBA
end

-- Create a new StarLibRGBAText object
-- @name WidgetText.New
-- @usage WidgetText:New(visitor, name, config, row, col, layer, type, environment, errorLevel)
-- @param this The inheritance object
-- @param visitor An StarLibCore-1.0 object
-- @parma name This widget's name
-- @param config This widget's configuration
-- @param row This widget's row
-- @param col This widget's column
-- @param layer This widget's layer
-- @param type Dict of widget types.
-- @param environment The code execution environment.
-- @param errorLevel The error level for this object.
-- @return A new StarLibRGBAText object
function LibRGBA:New(r, g, b, a) 
	
	local obj = next(pool)

	if obj then
		pool[obj] = nil
	else
		obj = {}
	end
					
	setmetatable(obj, self)

	obj.r = r or 0xff
	obj.g = g or 0xff
	obj.b = b or 0xff
	obj.a = a or 0xff

	return obj	
end


-- @name StarLibRGBA:Del
-- @usage StarLibRGBA:Del([widget]) or object:Del()
-- @param widget An optional widget object
-- @return Nothing
function LibRGBA:Del(widget)
	if type(widget) ~= "table" then widget = self end
	pool[widget] = true
end

-- @name StarLibRGBA.Compare
-- @usage STarLibRGBA:Compare(other)
-- @param other The other StarLibRGBA object to compare self to
-- @return True if the two colors (self and other) compare equally
function LibRGBA:Compare(other)
	local equal = true
	if self.R ~= other.R or other.G ~= v.G or self.B ~= other.B or self.A ~= other.A then
		equal = false
	end
	return equal
end

local function rgb2int(red, green, blue)
	return bit.lshift(red, 16) + bit.lshift(green, 8) + blue;
end

function LibRGBA:RedThroughBlue()
	local colors = {}

	-- Start with solid red:
	local r = 255;
	local g = 0;
	local b = 0;
	
	-- From red to yellow:
	local i = 0
	while i <= 255 do
		g = i
		tinsert(colors, {r / 255, g / 255, b / 255})
		i = i + 1
	end

	-- From yellow to green:
	i = 255
	while i >= 0 do
		r = i
		tinsert(colors, {r / 255, g / 255, b / 255})
		i = i - 1
	end
	
	-- From green to blue:
	i = 0
	while i <= 255 do
		b = i
		tinsert(colors, {r / 255, g / 255, b / 255})
		g = g - 1
		i = i + 1
	end

	local subColors = {}
	
	for i = #colors, 1, -1 do
		if mod(i, 8) == 0 then
			tinsert(subColors, colors[i])
		end
	end
	
	for i = #subColors + 1, 100 do
		tinsert(subColors, colors[1])
	end

	return subColors
end

